<?php

namespace WHMCS\Module\Server\VirtFusionResourcePack;

use WHMCS\Database\Capsule as DB;

class Database
{
    const SYSTEM_TABLE = 'mod_virtfusion_res_pks';

    /**
     * @return void
     */
    public static function schema(): void
    {
        if (!DB::schema()->hasTable(self::SYSTEM_TABLE)) {
            try {
                DB::schema()->create(self::SYSTEM_TABLE, function ($table) {
                    $table->unsignedBigInteger('service_id')->nullable()->default(null)->index();
                    $table->unsignedBigInteger('pack_id')->nullable()->default(null)->index();
                    $table->timestamps();
                });
            } catch (\Exception $e) {
                Log::insert(__FUNCTION__, [], $e->getMessage());
            }
        }
    }

    /**
     * @param int $server
     * @param $any
     * @return mixed
     */
    public static function getWhmcsServer(int $server, $any = false): mixed
    {
        if ($server) {
            return DB::table('tblservers')->where('type', 'VirtFusionResourcePack')->where('id', $server)->first();
        }

        if ($any) {
            return DB::table('tblservers')->where('type', 'VirtFusionResourcePack')->where('disabled', 0)->first();
        }

        return false;
    }

    /**
     * @param int $serviceId
     * @param int $userId
     * @return mixed
     */
    public static function userWhmcsService(int $serviceId, int $userId): mixed
    {
        return DB::table('tblhosting')->where('id', $serviceId)->where('userid', $userId)->exists();
    }

    /**
     * @return mixed
     */
    public static function getSystemUrl(): mixed
    {
        $url = DB::table('tblconfiguration')->where('setting', '=', 'SystemURL')->first();
        return $url->value;
    }

    /**
     * @param int $id
     * @return mixed
     */
    public static function getUser(int $id): mixed
    {
        return DB::table('tblclients')->where('id', $id)->first();
    }

    /**
     * @param int $serviceId
     * @return mixed
     */
    public static function getWhmcsService(int $serviceId): mixed
    {
        return DB::table('tblhosting')->where('id', $serviceId)->first();
    }

    /**
     * @param int $serviceId
     * @param int $packId
     * @return void
     */
    public static function updateSystemServicePackId(int $serviceId, int $packId): void
    {

        DB::table(self::SYSTEM_TABLE)->updateOrInsert(
            [
                "service_id" => $serviceId
            ],
            [
                'pack_id' => $packId
            ]
        );
    }

    /**
     * @param int $serviceId
     * @param $data
     * @return void
     */
    public static function updateWhmcsServiceParams(int $serviceId, $data): void
    {
        if (count($data)) {
            foreach ($data as $key => $items) {
                DB::table($key)->where('id', $serviceId)->update($items);
            }
        }
    }

    /**
     * @param int $serviceId
     * @return mixed
     */
    public static function checkSystemService(int $serviceId): mixed
    {
        return DB::table(self::SYSTEM_TABLE)->where('service_id', $serviceId)->exists();
    }

    /**
     * @param int $serviceId
     * @return void
     */
    public static function deleteSystemService(int $serviceId): void
    {
        DB::table(self::SYSTEM_TABLE)->where('service_id', $serviceId)->delete();
    }


    /**
     * @param int $serviceId
     * @param $data
     * @return void
     */
    public static function systemOnPackAdd(int $serviceId, $data): void
    {
        if (DB::table(self::SYSTEM_TABLE)->where('service_id', $serviceId)->exists()) {
            DB::table(self::SYSTEM_TABLE)->where('service_id', $serviceId)->update(['pack_id' => $data->data->id]);
        } else {
            DB::table(self::SYSTEM_TABLE)->insert(['service_id' => $serviceId, 'pack_id' => $data->data->id]);
        }
    }

    /**
     * @param int $serviceId
     * @return mixed
     */
    public static function getSystemService(int $serviceId): mixed
    {
        return DB::table(self::SYSTEM_TABLE)->where('service_id', $serviceId)->first();
    }

}